# from pycsclib import *
#import sys
#import tracemalloc
#import gc
#import csclib
#csclib.start(-1)
#from src.csclib.pycsclib import *

#from ccsclib import Csclib_core, Csclib_bits, Csclib_start
from ccsclib import *
_party = -1

class Share(Csclib_core):
    def typecheck(self, x):
        if not(type(x) is Share):
            raise Exception(x, "is not share")
    def __init__(self, array=None, order=None):
    #    print("Csclib.init")
        super().__init__()
        if array != None:
            if order == None:
                print('order is not given.')
            self.array(array, order)
        return None
    def print(self):
        print('n =', len(self), 'q =', self.order(), ':', self.get())
    def __str__(self):
        return self.str()
    def A2B(self, q):
    #    print("A2B")
        tmp = super().A2B(q)
        return Bits()._copy2(tmp)
    def __len__(self):
        return self.len()
    def __add__(self, other):
        if type(other) is int:
            #if self.len() != 1:
            #    print("add: length of self is ", self.len())
            #return self.addpublic(0, other)            
            other = Share([other] * self.len(), self.order())
            return self.vadd(other)
        elif type(other) is Share:
            return self.vadd(other)
        elif type(other) is list:
            #tmp = Share(other, self.order()) # addpublic を使うべき
            #return self.vadd(tmp)
            ans = self.dup()
            i = 0
            while i < self.len():
                ans = ans.addpublic(i, other[i]) # 毎回 dup するので非効率
                i += 1
            return ans
        else:
            print("add ???")
            #return self.vadd(other)
    def __radd__(self, other):
        if type(other) is int:
            #if self.len() != 1:
            #    print("add: length of self is ", self.len())
            #return self.addpublic(0, other)            
            other = Share([other] * self.len(), self.order())
            return self.vadd(other)
        elif type(other) is Share:
            return self.vadd(other)
        elif type(other) is list:
            #tmp = Share(other, self.order()) # addpublic を使うべき
            #return self.vadd(tmp)
            ans = self.dup()
            i = 0
            while i < self.len():
                ans = ans.addpublic(i, other[i])
                i += 1
            return ans

        else:
            print("radd ???")
            #return self.vadd(other)
    def __sub__(self, other):
        if type(other) is int:
            #if self.len() != 1:
            #    print("sub: length of self is ", self.len())
            #return self.addpublic(0, -other)            
            other = Share([other] * self.len(), self.order())
            return self.vsub(other)
        elif type(other) is Share:
            return self.vsub(other)
        elif type(other) is list:
            #tmp = Share(other, self.order()) # addpublic を使うべき
            #return self.vsub(tmp)
            ans = self.dup()
            i = 0
            while i < self.len():
                ans = ans.subpublic(i, other[i])
                i += 1
            return ans
    def __rsub__(self, other):
        if type(other) is int:
            #if self.len() != 1:
            #    print("sub: length of self is ", self.len())
            #tmp = self.vneg() # tmp = 1-self
            #return tmp.addpublic(0, other-1) # 1-self + other-1 = other-self
            other = Share([other] * self.len(), self.order())
            #return self.vsub(other)
            return other.vsub(self)
        elif type(other) is list:
            #tmp = Share(other, self.order()) # addpublic を使うべき
            #return tmp.vsub(self)
            ans = self.vneg()
            i = 0
            while i < self.len():
                ans = ans.addpublic(i, other[i]-1) # 正しい?
                i += 1
            return ans
        else:
            print("rsub ???")
            #return other.vsub(self)
    def __mul__(self, other):
        if type(other) is int:
            return self.smul(other)
        elif type(other) is Share:
            return self.vmul(other)
        elif type(other) is list:
            tmp = Share(other, self.order())
            return self.vmul(tmp)
        else:
            print("mul ???")
            #return self.vmul(other)
    def __rmul__(self, other):
        if type(other) is int:
            return self.smul(other)
        elif type(other) is Share:
            return self.vmul(other)
        elif type(other) is list:
            tmp = Share(other, self.order())
            return tmp.vmul(other)
        else:
            print("rmul ???")
            #return self.vmul(other)
    def __matmul__(self, other): # a @ b
        if type(other) is int:
            return self.insert_tail(other)
        elif type(other) is Share:
            return self.concat(other)
        elif type(other) is list:
            tmp = Share(other, self.order())
            return self.concat(tmp)
        else:
            print("@ ???")
            #return self.concat(other)
    def __rmatmul__(self, other): # a @ b
        if type(other) is int:
            return self.insert_head(other)
        elif type(other) is list:
            tmp = Share(other, self.order())
            return tmp.concat(self)
        else:
            print("rmatmul ???")
#    def __pow__(self, other): # a ** 0
#        return self.insert_tail(other)
#    def __rpow__(self, other): # 0 ** a
#        return self.insert_head(other)
############################################
# 論理演算
# 入力は 0, 1 のみ
############################################
    def __invert__(self): # ~a
        return self.vneg()
    def __and__(self, other): # a & b
        if type(other) is int:
            other = Share([other]*self.len(), self.order())
        return self.vmul(other)
    def __or__(self, other): # a | b
        if type(other) is int:
            other = Share([other]*self.len(), self.order())
        ap = self.vneg()
        bp = other.vneg()
        c = ap.vmul(bp)
        return c.vneg()
    def __xor__(self, other): # a ^ b
        if type(other) is int:
            other = Share([other]*self.len(), self.order())
        if self.order() == 2:
            ans = self.vadd(other)
        else:
            a = self.vadd(other)
            c = self.vmul(other)
            d = c.smul(2)
            ans = a.vsub(d)
        return ans
############################################
# 比較
    def __eq__(self, other):
        if self.order() != 2 or other.order() != 2:
            self2 = self.A2B(2)
            other2 = other.A2B(2)
        #    self2 = self.A2B(self.order())
        #    other2 = other.A2B(self.order())
            return self2 == other2
        else:
            return self.eq(other)
    def __ne__(self, other):
        if self.order() != 2 or other.order() != 2:
            self2 = self.A2B(2)
            other2 = other.A2B(2)
            return self2 != other2
        else:
            return self.eq(other).vneg()
    def __lt__(self, other):
    #    print("Share lt")
        if type(other) is int:
            other = Share([other]*self.len(), self.order())
        if self.order() != 2 or other.order() != 2:
            self2 = self.A2B(2)
            other2 = other.A2B(2)
            return self2 < other2
        else:
            return self.lt(other)
    def __ge__(self, other):
        if self.order() != 2 or other.order() != 2:
            self2 = self.A2B(2)
            other2 = other.A2B(2)
            return self2 >= other2
        else:
            return self.lt(other).vneg()
    def __gt__(self, other):
    #    print("Share gt")
    #    print("self")
    #    self.print()
    #    print("other")
    #    other.print()
        if type(other) is int:
            other = Share([other]*self.len(), self.order())
        if self.order() != 2 or other.order() != 2:
            self2 = self.A2B(2)
            other2 = other.A2B(2)
        #    self2 = self.A2B(self.order())
        #    other2 = other.A2B(self.order())
            return self2 > other2
        else:
            return other.lt(self)
    def __le__(self, other):
        if self.order() != 2 or other.order() != 2:
            self2 = self.A2B(2)
            other2 = other.A2B(2)
            return self2 <= other2
        else:
            return other.lt(self).vneg()
    def __bool__(self):
        if self.len() != 1:
            print("bool: length of self is ", self.len())
        c0 = self.reconstruct()
#        print("bool c0")
#        c0.print()
        c1 = c0.get()
        c = c1[0]
        return c == 1
    
############################################
    def __getitem__(self, index):
        #print('getitem slice', index, type(index))
        if type(index) is int:
            return self.slice(index, index+1)
        elif type(index) is slice:
            start = index.start
            end = index.stop
            #print('start', start, 'end', end, 'step', index.step)
            if start == None:
                start = 0
            if end == None:
                end = 0
            return self.slice(start, end)
        else:
            print("getitem?")
    def __setitem__(self, index, x):
        #print('setitem slice', index, type(index))
        #x.print()
        if type(index) is int: # a[i] = x
            if type(x) is int: # 公開値
                return self.setpublic(index, x)
            else:
                if x.len() != 1:
                    print("setitem: length of x is ", x.len())
                return self.setshare(index, x, 0)
        elif type(index) is slice: # a[s:e] = x
            start = index.start
            end = index.stop
            if type(x) is int: # 公開値
            #    return self.setpublic(start, x) # うそ
                i = start
                while i < end:
                    self.setpublic(i, x)
                    i += 1
                return self
            else:
                if start == None:
                    start = 0
                if end == None:
                    end = len(self)
                return self.setshares(start, end, x, 0)
    def sort(self):
        return radix_sort(self)
##############################################
# Share の定義おわり
##############################################

class Bits(Csclib_bits):
    def typecheck(self, x):
        if not(type(x) is Bits):
            raise Exception(x, "is not bit share")
    def __init__(self, array=None, order=None, order2=None):
        #print("Bits(Csclib_bits) init")
        super().__init__()
        if array != None:
            if order == None or order2 == None:
                print('order is not given.')
            tmp = Share().array(array, order)
            tmp2 = tmp.A2B(order2) # order2 は変換後の各桁の位数
            self._copy2(tmp2)
        return None
    def print(self):
        tmp = super().get()
        i = 0
        for x in tmp:
            print('i =', i, end=" ")
            Share()._copy(x).print()
            i += 1
    def __str__(self):
        return self.str()
    def get(self):
        tmp = super().get()
        ans = []
        for x in tmp:
            ans.append(Share()._copy(x))
        return ans
    def new(self, list):
        tmp = super().set(list)
        return Bits()._copy(tmp)
    def __pow__(self, other): # X ** Y (下位ビットが X, 上位ビットが Y)
        return Bits().new(self.get() + other.get())
    def __len__(self):
        return self.len()
    def B2A(self):
        tmp = super().B2A()
        return Share()._copy(tmp)
    def __getitem__(self, index):
        #print('slice', index, type(index))
        if type(index) is int:
            return self.slice(index, index+1) 
        elif type(index) is slice:
            start = index.start
            end = index.stop
            #print('start', start, 'end', end, 'step', index.step)
            if start == None:
                start = 0
            if end == None:
                end = 0
            return self.slice(start, end)
        else:
            print("getitem?")
    def __setitem__(self, index, x):
        if type(index) is int: # a[i] = x
            if type(x) is int: # 公開値
                return self.setpublic(index, x)
            else:
                if x.len() != 1:
                    print("setitem: length of x is ", x.len())
                return self.setshare(index, x, 0)
        elif type(index) is slice: # a[s:e] = x
            start = index.start
            end = index.stop
            if type(x) is int: # 公開値
                i = start
                while i < end:
                    self.setpublic(i, x)
                    i += 1
                return self
        else:
            print("setitem?")
############################################
# 比較
    def __eq__(self, other):
        tmp = super().eq(other)
        return Share()._copy(tmp)
    def __ne__(self, other):
        tmp = super().eq(other)
        return Share()._copy(tmp.vneg())
    def __lt__(self, other):
    #    print("Bits lt")
    #    print("self")
    #    self.print()
    #    print("other")
    #    other.print()
        tmp = super().lt(other)
        return Share()._copy(tmp)
    def __ge__(self, other):
        tmp = super().lt(other)
        return Share()._copy(tmp.vneg())
    def __gt__(self, other):
#        print("Bits gt")
#        print("other")
#        other.print()
#        print("self")
#        self.print()
        tmp = other.lt(self)
        return Share()._copy(tmp)
    def __le__(self, other):
        tmp = other.lt(self)
        return Share()._copy(tmp.vneg())
    def sort(self):
        return radix_sort_bits(self)

##############################################
# Bits の定義おわり
##############################################

############################################
# 整数の桁数を求める
# to store an integer in [0,x-1], we need blog(x-1)+1 bits
############################################
def blog(x):
    l = -1
    while x > 0:
        x >>= 1
        l += 1
    return l

################################
# x を下位 low ビットとそれ以外に分ける
################################
def partition(x, low):
    q = x.order()
    B = x.A2B(2).get()
    R = B[:low] # 下位
    Q = B[low:] # 上位
    i = 0
    while i < len(R):
        R[i] = R[i].extend(1<<low)
        i += 1
    i = 0
    while i < len(Q):
        Q[i] = Q[i].extend(q)
        i += 1
    rb = Bits().new(R)
    qb = Bits().new(Q)

    L = rb.B2A()
    H = qb.B2A()
    return (H, L)

def PrefixSum(v_):
    n = len(v_)
#    if v_.order() < n+1:
#        k2 = blog(n+1-1)+1
#        v = v_.extend(1<<k2)
#    else:
#        v = v_.dup()
    v = v_.dup()

#    ans = v.dup()
#    sum = 0
#    i = 0
#    while i < n:
#        sum += v[i]
#        ans[i] = sum
#        i += 1
    ans = v.PrefixSum()
    return ans

def SuffixSum(v_):
    n = len(v_)
#    if v_.order() < n+1:
#        k2 = blog(n+1-1)+1
#        v = v_.extend(1<<k2)
#    else:
#        v = v_.dup()
    v = v_.dup()


#    ans = v.dup()
#    sum = 0
#    i = n-1
#    while i >= 0:
#        sum += v[i]
#        ans[i] = sum
#        i -= 1
    ans = v.SuffixSum()
    return ans

def Diff(v, z):
    n = len(v)
#    ans = v.dup()
#    prev = z
#    i = 0
#    while i < n:
#        ans[i] = v[i] - prev
#        prev = v[i]
#    return ans
    return v.Diff(z)

def rank1(v_):
    n = len(v_)
    if v_.order() < n+1:
        k2 = blog(n+1-1)+1
        v = v_.extend(1<<k2)
    else:
        v = v_.dup()
#    ans = v.dup()
#    sum = 0
#    i = 0
#    while i < n:
#        sum += v[i]
#        ans[i] = sum
#        i += 1
    ans = v.rank1()
    return ans

def rank0(v_):
    n = len(v_)
    if v_.order() < n+1:
        k2 = blog(n+1-1)+1
        v = v_.extend(1<<k2)
    else:        
        v = v_.dup()
#    ans = v.dup()
#    sum = 0
#    i = 0
#    while i < n:
#        sum += 1 - v[i]
#        ans[i] = sum
#        i += 1
    ans = v.rank0()
    return ans

def Sum(v_):
    n = len(v_)
    if v_.order() < n+1:
        k2 = blog(n+1-1)+1
        v = v_.extend(1<<k2)
    else:
        v = v_.dup()
#    ans = v[0]
#    i = 1
#    while i < n:
#        ans[0] += v[i]
#        i += 1
    ans = v.sum()
    return ans

def rshift(v, z):
#    n = len(v)
#    ans = v.dup()
#    ans = Share().const(n, 0, v.order())
#    ans.setshares(0, n, v, 0)
#    ans.setshares(1, n, v, 0)
#    ans[0] = v.public(z)
#    ans.setpublic(0, z)
#    ans = v.public(z) ** v[:-1]
#    ans = v.public(z) @ v[:-1]
    ans = z @ v[:-1]
    return ans

def lshift(v, z):
#    n = len(v)
#    ans = v.dup()
#    ans.setshares(0, n-1, v, 1)
#    ans[n-1] = v.public(z)
#    ans = v[1:] ** v.public(z)
#    ans = v[1:] @ v.public(z)
    ans = v[1:] @ z
    return ans

def rrotate(v):
    n = len(v)
#    ans = v.dup()
#    ans.setshares(1, n, v, 0)
#    ans[0] = v[n-1]
#    ans = v[n-1] ** v[:-1]
    ans = v[n-1] @ v[:-1]
    return ans

def lrotate(v):
    n = len(v)
#    ans = v.dup()
#    ans.setshares(0, n-1, v, 1)
#    ans[n-1] = v[0]
#    ans = v[1:] ** v[0]
    ans = v[1:] @ v[0]
    return ans

def IfThenElse(f, a, b):
    if f.order() < a.order():
        f = f.extend(a.order())
    n = len(f)
    if n !=len(a) or n != len(b):
        print("IfThenElse f->n =", n, "a->n = ", len(a), "b->n = ", len(b))
    ans = (a - b)*f + b
    return ans

def StableSort(g_):
    if g_.order() < g_.len():
        k2 = blog(g_.len()-1)+1
        g = g_.extend(1<<k2)
    else:
        g = g_.dup()
    return g.StableSort()
#    print("g")
#    g.print()
    n = len(g)
    r0 = rank0(g)
#    print("r0")
#    r0.print()
    r1 = rank1(g)
#    print("r1")
#    r1.print()
    s0 = rshift(r0, 0)
#    print("s0")
#    s0.print()
    s1 = rshift(r1, 0)
#    print("s1")
#    s1.print()
#    s = r0[n-1]
#    i = 0
#    while i < n:
#        s1[i] += s
#        i += 1
    s1.addall(r0[n-1])
#    print("s1")
#    s1.print()
    sigma = IfThenElse(g, s1, s0)
#    print("sigma")
#    sigma.print()
    return sigma


def Perm_ID(v):
    n = len(v)
    ans = v.dup()
#    i = 0
#    while i < n:
#        ans[i] = i
#        i += 1
    ans.setperm()
    return ans

def Perm_ID2(n, q):
    w = blog(q-1)+1
    ans = Share().const(n, 0, 1<<w)
    return Perm_ID(ans)

def InvPerm(sigma):
    perm = Perm_ID(sigma)
    ans = perm.AppInvPerm(sigma)
    return ans


def GenCycle(g_):
    if g_.order() < g_.len():
        k2 = blog(g_.len()-1)+1
        g = g_.extend(1<<k2)
    else:
        g = g_.dup()
    return g.GenCycle()
    n = len(g)
    sigma = StableSort(g)
    perm = Perm_ID(g)
    gneg = ~g
    t0 = perm * gneg
    t1 = perm * g
    u = t1.AppInvPerm(sigma)
    v = lshift(u, 0)
    y = v.AppPerm(sigma)
    pi = y + t0
    w = rshift(u, 0)
    z = w.AppPerm(sigma)
    pi_inv = z + t0
    pi_inv[0] = u[n-1]
    return (pi, pi_inv)

def Propagate(g_, v):
    if g_.order() < g_.len():
        k2 = blog(g_.len()-1)+1
        g = g_.extend(1<<k2)
    else:
        g = g_.dup()
    (pi, tmp_y) = GenCycle(g)
#    print("pi")
#    pi.print()
#    print("tmp_y")
#    tmp_y.print()

    x = v.AppInvPerm(pi)
#    print("x0")
#    x.print()
    x[0] = 0
#    print("x1")
#    x.print()
    v2 = v - x
 #   print("v2")
 #   v2.print()
    z = PrefixSum(v2)
 #   print("z")
 #   z.print()
    return z

def GroupSum(g_, v):
    if g_.order() < g_.len():
        k2 = blog(g_.len()-1)+1
        g = g_.extend(1<<k2)
    else:
        g = g_.dup()
    (tmp_x, pi_inv) = GenCycle(g)
    s = SuffixSum(v)
    t = s.dup()
    t[0] = 0
    y = t.AppInvPerm(pi_inv)
    s = s - y
    return s

def GroupSumBlock(g_, v, block_size):
    if g_.order() < g_.len():
        k2 = blog(g_.len()-1)+1
        g = g_.extend(1<<k2)
    else:
        g = g_.dup()
    (tmp_x, pi_inv) = GenCycle(g)
    #s = SuffixSum(v)
    n = len(g)
    s = v.dup()
    sum = Share([0]*block_size, v.order())
    i = n
    while i > 0:
        sum += v[(i-1)*block_size:i*block_size]
        s[(i-1)*block_size:i*block_size] = sum
        i -= 1

    t = s.dup()
    t[0:block_size] = 0
    y = t.AppInvPermBlock(block_size, pi_inv)
    s = s - y
    return s

def select1(g_):
    if g_.order() < g_.len():
        k2 = blog(g_.len()-1)+1
        g = g_.extend(1<<k2)
    else:
        g = g_.dup()
    return g.select1()
    n = len(g)
    s0 = rank0(g)
    s0 = rshift(s0, 0)
    m = sum(g)
#    i = 0
#    while i < n:
#        s0[i] += m[0]
#        i += 1
    s0.addall(m)
    s1 = rank1(g)
    s1 = rshift(s1, 0)
    sigma = IfThenElse(g, s1, s0)
    gneg = ~g
    t0 = n * gneg
    t1 = g.dup()
    i = 0
    while i < n:
        t1[i] = t1[i] * i ############################# 要改良
        i += 1
    t = t0 + t1
    u = t.AppInvPerm(sigma)
    return u

def select0(g_):
    if g_.order() < g_.len():
        k2 = blog(g_.len()-1)+1
        g = g_.extend(1<<k2)
    else:
        g = g_.dup()
    return g.select0()

def radix_sort(a_):
    a = a_.dup()
    w = blog(len(a)-1)+1
    pi = Perm_ID2(len(a), 1 << w)
    q = a.order()
    qb = 1 << w
    k = 1
    while k < q:
#        print("k ", k)
#        (tmp_x, tmp_y) = a.A2QB(q//k, 2)
        (tmp_x, tmp_y) = a.A2QB(q//k, q)
        sigma = StableSort(tmp_y)
        if tmp_x.order() > 1:
            a = tmp_x.AppInvPerm(sigma)
        pi = pi.AppInvPerm(sigma)
        k *= 2
    x = a_.AppPerm(pi)
    return (x, pi)


def radix_sort_bits(a):
    d = a.depth()
 #   print("a", a)
    A = a.get()
 #   print("A", A)
    pi = StableSort(A[0])
    k = 1
    while k < d:
        ap = A[k].AppInvPerm(pi)
        sigma = StableSort(ap)
        pi = sigma.AppPerm(pi)
        k += 1
    return pi


def Grouping_bit(V):
    Vp = rshift(V, 1)
    Vp[0] -= V[0]
    ans = ~(V == Vp)
    return ans

def Grouping_bits(V):
    B = V.get()
    d = len(B)
    b = Grouping_bit(B[0])
    i = 1
    while i < d:
        g = Grouping_bit(B[i])
        b = b | g
        i += 1
    return b

def Propagate_bits(g, V):
#    print("g")
#    g.print()
    B = V.get()
    ans = []
    for b in B:
#        print("b")
#        b.print()
#        print("tmp")
        tmp = Propagate(g, b)
#        tmp.print()
        ans.append(Propagate(g, b))
    return Bits().new(ans)

def Grouping_name(L, q):
    V = Share([0] * L.len(), q)
#    i = 0
#    while i < V.len():
#        V[i] = i
#        i += 1
    V.setperm()
    return Propagate(L, V)


def BatchAccessUnary(v, idx):
    U = len(v)
    N = len(idx)
    sigma = StableSort(idx)
#    n = Sum(idx)
    zeros = Share([0]*(N-U), v.order())
    X = v @ zeros
    Y = X.AppPerm(sigma)
    nidx = ~idx
    Z = Propagate(nidx, Y)
    W = Z.AppInvPerm(sigma)
    ans = W[U:N] # W[U:]
    return ans



#__version__ = '202601291'

#from ccsclib import Csclib_core, Csclib_bits, Csclib_start
#from src.csclib.pycsclib import Share as Share
#from src.csclib.pycsclib import Bits as Bits
#from ccsclib import *


from .pycsclib import *
from .SA import *
from .LOUDS import *
#Csclib_start(-1)
#print("csclib.init2")
_party = -1



from csclib import *

def SuffixSort(T):
    #print("T")
    #T.print()
    n = len(T)
    d = blog(n+1-1)+1
#    allone = Share([1]*n, T.order())
    allone = Share().const(n, 1, T.order())
    V = T + allone
    V = V @ 0
    Vb = V.A2B(1<<d) # T をビット分解する
#    Vb = V.A2B(T.order()) # T をビット分解する
#    J = radix_sort_bits(Vb) #  1文字でのソートを表す置換
    J = Vb.sort() #  1文字でのソートを表す置換
#    print('J')
#    J.print()
    W = Vb.AppInvPerm(J) # 1文字のソート結果 (pos -> lex)
#    print('W')
#    W.print()
    L = Grouping_bits(W) # グループの境界を求める

#    Va = Bits([0]*(n+1), 1<<d, 1<<d)
#    i = 0
#    while i < n+1:
#        Va[i] = i
#        i += 1
#    Va = Propagate_bits(L, Va)
    Va = Grouping_name(L, 1<<d)
    Va = Va.A2B(1<<d)

    h = 1
    s = Sum(L)
    while h < n:
    #    print("h", h)
        V1 = Va.AppPerm(J)
    #    J2 = Share().const(n+1, 0, 1<<d)
#        J2 = Share([0]*(n+1), 1<<d)
#        i = 0
#        while i < n+1:
#            J2[i] = J[(i+h) % (n+1)]
#            i += 1
        J2 = J[h:] @ J[:h]
        V2 = Va.AppPerm(J2)
        #tmp = V2.get() + V1.get()
        #Vb = Bits().new(tmp)
        Vb = V2 ** V1
    #    J = radix_sort_bits(Vb)
        J = Vb.sort()
        W = Vb.AppInvPerm(J)
        L = Grouping_bits(W)
#        i = 0
#        while i < n+1:
#            Va[i] = i
#            i += 1
#        Va = Propagate_bits(L, Va)
        Va = Grouping_name(L, 1<<d)
        Va = Va.A2B(1<<d)


        h = h*2
        s = Sum(L)
        print("s")
        s.print()
#        sn = Share([n], s.order())
#        c = (s > sn)
#        c = (s > n)
#        c.print()
#        if bool(s > sn):
        if bool(s > n):
#            print("break")
            break
#    print("J")
#    J.print()
    I = InvPerm(J)
#    print("I")
#    I.print()
    return I

def SuffixSort_LCP(T):
    #print("T")
    #T.print()
    n = len(T)
    d = blog(n+1-1)+1
#    allone = Share().const(n, 1, T.order())
    allone = Share([1]*n, T.order())
    V = T + allone
    V = V @ 0
    Vb = V.A2B(1<<d) # T をビット分解する
#    J = radix_sort_bits(Vb) #  1文字でのソートを表す置換
    J = Vb.sort() #  1文字でのソートを表す置換
    W = Vb.AppInvPerm(J) # 1文字のソート結果 (pos -> lex)
    L = Grouping_bits(W) # グループの境界を求める
#    Va = Bits([0]*(n+1), 1<<d, 1<<d)
#    i = 0
#    while i < n+1:
#        Va[i] = i
#        i += 1
#    Va = Propagate_bits(L, Va)
    Va = Grouping_name(L, 1<<d)
    Va = Va.A2B(1<<d)

    VV = []
    VV.append(Va.dup())

    h = 1
    s = Sum(L)
    while h < n:
    #    print("h", h)
        V1 = Va.AppPerm(J)
    #    J2 = Share().const(n+1, 0, 1<<d)
#        J2 = Share([0]*(n+1), 1<<d)
#        i = 0
#        while i < n+1:
#            J2[i] = J[(i+h) % (n+1)]
#            i += 1
        J2 = J[h:] @ J[:h]
        V2 = Va.AppPerm(J2)
#        tmp = V2.get() + V1.get()
#        Vb = Bits().new(tmp)
        Vb = V2 ** V1
#        J = radix_sort_bits(Vb)
        J = Vb.sort()
        W = Vb.AppInvPerm(J)
        L = Grouping_bits(W)
#        i = 0
#        while i < n+1:
#            Va[i] = i
#            i += 1
#        Va = Propagate_bits(L, Va)
        Va = Grouping_name(L, 1<<d)
        Va = Va.A2B(1<<d)

        h = h*2
        VV.append(Va.dup())
        s = Sum(L)
        print("s")
        s.print()
        if bool(s > n):
            break


    I = InvPerm(J)

    LCP = Share([0]*n, 1<<d)

    d = len(VV)-2
    while d >= 0:
        V = VV[d].AppPerm(J)
    #    I1 = I[0:n]
    #    I2 = I[1:n+1]
    #    i = 0
    #    while i < n:
    #        I1[i] = I[i] + LCP[i]
    #        I2[i] = I[i+1] + LCP[i]
    #        i += 1
        I1 = I[0:n] + LCP
        I2 = I[1:n+1] + LCP
        V1 = V.BatchAccess(I1) # V1[i] := V[I1[i]] = V[I[i]  +LCP[i]]
        V2 = V.BatchAccess(I2) # V2[i] := V[I2[i]] = V[I[i+1]+LCP[i]]
    #    print("V1 ")
    #    V1.print()
    #    print("V2 ")
    #    V2.print()
        c = (V1 == V2)
    #    print("c ")
    #    c.print()
        c = c * (1<<d)
        LCP += c
        d -= 1

    return (I, LCP)

def merge(X, Y, rank_Y):
    if len(Y) != len(rank_Y):
        print('merge')
        Y.print()
        rank_Y.print()
        return None
    U = rank_Y.Unary(len(X)+1)[1:]
    pi = StableSort(U)
    Z = X @ Y
    ans = Z.AppPerm(pi)
    return ans

def getrank(sigma):
    n2 = sigma.len()//2
    q = sigma.order()
    B = Share([0]*n2 + [1]*n2, q)
    Ba = B.AppInvPerm(sigma)
    Bb = PrefixSum(Ba)
    Bc = Bb.AppPerm(sigma)
    r  = Bc[:n2]
    return r


def SuffixSort_DC3(T):
    n = len(T)
    print('SuffixSort_DC3 n = ', n, 'order = ', T.order())

    if n < 10:
        return SuffixSort(T)

# T12 を作る
    n2 = (n+3)//3
    o = T.order() + 3
    k = max(o, n+1) # !!!
    d = blog(k-1)+1

    T12 = [None] * 3
    d2 = max(d, blog(o+1)+1)
    Tp = T.extend(1<<d2)
    i = 0
    while i < n:
        Tp[i] = Tp[i] + 3
        i += 1
    Tp = Tp @ [2,1,0,0,0]


    j = 0
    while j < 3:
        T12[j] = Share([0] * (n2*2), 1<<d2)
#        T12[j] = Share().const(n2*2, 0, 1<<d2)
        j += 1
    i = 0
    while i < n2:
        j = 0
        while j < 3:
            T12[j][i] = Tp[i*3+1+2-j]
            j += 1
    #    T12pos[i] = i*3+1
        i += 1
    i = 0
    while i < n2:
        j = 0
        while j < 3:
            T12[j][n2+i] = Tp[i*3+2+2-j]
            j += 1
    #    T12pos[n2+i] = i*3+2
        i += 1
    T12a = [None] * 3
    j = 0
    while j < 3:
        T12a[j] = T12[j].A2B(1<<d2)
        j += 1
    T12b = T12a[0] ** T12a[1] ** T12a[2]
    J = T12b.sort() #  1文字でのソートを表す置換
#    print('J')
#    J.print()

    Wb = T12b.AppInvPerm(J) # 1文字のソート結果 (pos -> lex)
#    print('Wb')
#    Wb.print()
    L = Grouping_bits(Wb) # グループの境界を求める
#    print('L')
#    L.print()

    d3 = blog(len(L)+2-1)+1
    V = Grouping_name(L, 1<<d3)
#    V = rank1(L)
#    print('V')
#    V.print()

    T12c = V.AppPerm(J)
 #   print('T12c')
 #   T12c.print()
    T12c = T12c[:-1]

#    SA12 = SuffixSort(T12c)
    SA12 = SuffixSort_DC3(T12c)
#    print('SA12')
#    SA12.print()
#    s = L.extend(1<<d3).sum().get()[0]
#    print('sum', s)
#    if s < len(T12c)+1:
#        SA12 = SuffixSort_DC3(T12c)
#    else:
#        SA12 = InvPerm(J)


    SA12 = SA12.extend(1<<d)
    SA12inv = InvPerm(SA12)
#    print('SA12inv')
#    SA12inv.print()

    n02 = n2*2

    T01 = Share([0] * n02, Tp.order())
    SA01 = Share([0] * n02, SA12inv.order())
#    T01 = Share().const(n02, 0, Tp.order())
#    SA01 = Share().const(n02, 0, SA12inv.order())
    i = 0
    while i < n2:
        T01[i] = Tp[i*3+0]
        SA01[i] = i*3+0
        i += 1
    i = 0
    while i < n2:
        T01[n2+i] = Tp[i*3+1]
        SA01[n2+i] = i*3+1
        i += 1

    T01x = T01.AppInvPerm(SA12inv)
    T01xb = T01x.A2B(1<<d2)
    sigma3 = T01xb.sort()
    sigma = sigma3.AppPerm(SA12inv)


    B01 = Share([0]*n2 + [1]*n2, 1<<d)
    B01a = B01.AppInvPerm(sigma)  # 後で使う
    B01b = PrefixSum(B01a)
    B01c = B01b.AppPerm(sigma)
    r1 = B01c[:n2] # rank
#    r1 = getrank(sigma)

    T021 = Share([0] * (n02), Tp.order())
    T022 = Share([0] * (n02), Tp.order())
    R02 = Share([0] * (n02), SA12inv.order())
    i = 0
    while i < n2:
        T021[i] = Tp[i*3+0]
        T022[i] = Tp[i*3+1]
        R02[i] = SA12inv[n2+i]
        i += 1
    i = 0
    while i < n2:
        T021[n2+i] = Tp[i*3+2]
        T022[n2+i] = Tp[i*3+3]
        if i+1 < n2:
            R02[n2+i] = SA12inv[i+1]
        i += 1
    R02[n2*2-1] = SA12inv[0]
    T021b = T021.A2B(1<<d2)
    T022b = T022.A2B(1<<d2)

    K02c = T022b ** T021b
    T02x = K02c.AppInvPerm(R02)
    sigma4 = T02x.sort()
    sigma2 = sigma4.AppPerm(R02)


    B02 = Share([0]*n2 + [1]*n2, 1<<d)
    B02a = B02.AppInvPerm(sigma2)
    B02b = PrefixSum(B02a)
    B02c = B02b.AppPerm(sigma2)
    r2 = B02c[:n2]
#    r2 = getrank(sigma2)


    T12pos = Share([0] * (n2*2), 1<<d)
#    T12pos = Share().const(n2*2, 0, 1<<d)
    i = 0
    while i < n2:
        T12pos[i] = i*3+1
        T12pos[n2+i] = i*3+2
        i += 1
    SA12t = T12pos.AppPerm(SA12)

    rho2 = StableSort(B01a)
    SA01a = SA01.AppInvPerm(sigma)
    SA01t = SA01a.AppInvPerm(rho2)[:n2]

    R = r1 + r2
#    U = R.Unary(n2*2+1)[1:]
#    pi = StableSort(U)
#    SAtmp = SA12tmp2 @ SA01t
#    SA = SAtmp.AppPerm(pi)
    SA = merge(SA12t, SA01t, R)

    if SA.len() > n+1:
        SA = SA[SA.len()-(n+1):]

    return SA

def Suffix_print(T, SA):
    n = len(T)
    i = 0
    while i <= n:
        j = SA[i]
        print('SA[',i,'] = ', j, ' ', end='')
        while j < n:
            print(chr(T[j]), end='')
            j += 1
        print()
        i += 1



import sys
import random
import time
from csclib import *



if __name__ == '__main__':
  party = -1

  args = sys.argv
  if len(args) >= 3:
    party = int(args[1])
    fname = args[2]
  elif len(args) >= 2:
    fname = args[1]

  print('party ', party)
  Csclib_start(party)


  rB = [0,1,0,0,1,1,0,0,1,0,0,0,0,1,0,0]
  q = len(rB)*2
  B = Share(rB, q)

  n = len(B)
  m = Sum(B).reconstruct().get()[0]
  print(n, m)

  w = blog(n-1)+1
  z = blog(m)
  print(w, z)

  s1 = select1(B)[:m]
  s1.print()

#  sb = s1.A2B(2).get()
#  R = sb[:w-z]
#  Q = sb[w-z:]
#  i = 0
#  while i < len(R):
#    R[i] = R[i].extend(1<<(w-z))
#    i += 1
#  i = 0
#  while i < len(Q):
#    Q[i] = Q[i].extend(1<<z)
#    i += 1
#  rb = Bits().new(R)
#  qb = Bits().new(Q)
#
#  L = rb.B2A()
#  q = qb.B2A()
  (Q, L) = partition(s1, w-z)
  L.print()
  Q.print()

  U = Q.Unary(1<<z)
  H = U.A2QB(1<<z, 2)[1]
  H.print()
  L.print()



from csclib import *

def LOUDS_parent(b):
    n = len(b)
    p = rank1(b)
    sigma = StableSort(b)
    v = p.AppInvPerm(sigma)
    parent = v[0:n//2]
    i = 0
    while i < len(parent):
        parent[i] += -1
        i += 1
    return parent

def LOUDS_firstchild(bb):
    b = bb[1:]
    n = len(b)
    Q = rank0(b)
    sigma = StableSort(b)
    v = Q.AppInvPerm(sigma)
    firstchild = v[n//2: n]
    i = 0
    while i < len(firstchild):
        firstchild[i] += 1
        i += 1
    return firstchild

def LOUDS_contract(b_):
    b = b_[1:]
    n = len(b)
    p = rank1(b)
    q = rank0(b)
    sigma = StableSort(b)
    r = IfThenElse(b, q, p)
    v0 = r.AppInvPerm(sigma)
    v1 = v0.dup()
    v1[0:(n//2)-1] = v0[(n//2)+1:(n//2)*2]
    v1[(n//2)-1] = n//2
    v1[n//2:(n//2)*2+1] = v0[0:(n//2)+1]
    v  = v1.AppPerm(sigma)
    d1 = Propagate(~b, v)
    d1 -= r
    b1 = 1 @ b
    zv = 0 @ v
    d0 = Propagate(b1, zv)
    d0 = d0[1:]
    d0 -= r
    d = IfThenElse(b, d1, d0)
    perm = Perm_ID(b)
    pi = d + perm
    bb = 1 @ b.AppInvPerm(pi)
    return (bb, pi)

def LOUDS_grandparent(b):
    parent = LOUDS_parent(b)
    bb = b[1:]
    bneg = ~bb
    bneg = bneg[1:]
    p2 = 0 @ BatchAccessUnary(parent, bneg)
    return p2

def LOUDS_grandchild(b):
    n = len(b)//2
    firstchild = LOUDS_firstchild(b) @ (n+1)
    c2 = BatchAccessUnary(firstchild, 0 @ b)
    c2 = c2[1:]
    return c2

def select0_from_parent(parent):
    n = len(parent)
    pi = parent.dup()
    i = 0
    while i < n:
        pi[i] += i+1
    return pi

def select1_from_child(firstchild):
    n = len(firstchild)
    pi = firstchild.dup()
    i = 0
    while i < n:
        pi[i] += i
    return 0 @ pi

def ArrayToLOUDS(parent, firstchild):
    n = len(parent)
    pi1 = select0_from_parent(parent)
    pi2 = select1_from_child(firstchild)
    pi = pi1 @ pi2
    b = Share([0]*n + [1]*(n+1), parent.order())
    L = b.AppInvPerm(pi)
    return (L, pi)

def LOUDS_contract_supereasy(b):
    p2 = LOUDS_grandparent(b)
    c2 = LOUDS_grandchild(b)
    (L, pi) = ArrayToLOUDS(p2, c2)
    return L

def LOUDS_PathSum(b, w):
    n = len(b)//2
    b0 = b.dup()
    bb1 = b0[1:]
    sigma = StableSort(bb1)
    w2 = w @ w
    s0 = 0 @ w2.AppPerm(sigma)
    r = 1
    while r <= n:
        sigma0 = StableSort(b0)
        d0 = Propagate(b0, s0)
        s0 += d0
        e1 = s0.AppInvPerm(sigma0)
        (b1, rho0) = LOUDS_contract(b0)
        ztmp = e1[0:n]
        z1 = ztmp @ 0
        z1 = z1 @ ztmp
        c1 = z1.AppPerm(sigma0)
        c1 = c1[1:]
        s1 = c1.AppInvPerm(rho0)
        r = r*2
        b0 = b1
        s0 = 0 @ s1
    ans = z1[0:n]
    return ans

def LOUDS_get_parent(w, b_):
    b = b_[1:]
    b = 0 @ b
    b = 1 @ b
    I = ~b
    wp = BatchAccessUnary(w, I)
    return wp

def LOUDS_PathSum_supereasy(b_, w_):
    b = b_.dup()
    w = 0 @ w_
    n = len(w)
    d = 1
    while d < n:
        wp = LOUDS_get_parent(w, b)
        w += wp
        b = LOUDS_contract_supereasy(b)
        d = d*2
    w = w[1:]
    return w

def LOUDS_TreeSum(b, w):
    bb = b[1:]
    sigma = StableSort(bb)
    w2 = w @ w
    wb = w2.AppPerm(sigma)
    n = len(b)//2
    b0 = bb
    s0 = wb
    r = 1
    while r <= n:
        sigma0 = StableSort(b0)
        p0 = SuffixSum(s0)
        e1 = p0.AppInvPerm(sigma0)
        z20 = e1[n:]
        z21 = e1[n+1:] @ 0
        z2 = z20 - z21
        z1 = z2 @ z2
        b01 = 1 @ b0
        (b1, rho0) = LOUDS_contract(b01)
        c1 = z1.AppPerm(sigma0)
        s1 = c1.AppInvPerm(rho0)
        r = r*2
        b0 = b1[1:]
        s0 = s1
    ans = z1[0:n]
    return ans

def LOUDS_Distribute(wp, wc, b):
    sigma = StableSort(b)
    ww = wc @ wp
    wb = ww.AppPerm(sigma)
    return wb

def LOUDS_Duplicate(w, b):
    return LOUDS_Distribute(w, w, b)

def LOUDS_gather(wb, b):
    n = len(b)//2
    sigma0 = StableSort(b)
    e1 = wb.AppInvPerm(sigma0)
    e1 = e1[n:]
    return e1

def LOUDS_TreeSum_easy(b, w):
    bb = b[1:]
    wb = LOUDS_Duplicate(w, bb)
    n = len(b)//2
    b0 = bb
    s0 = wb
    r = 1
    while r <= n:
        p0 = SuffixSum(s0)
        e1 = LOUDS_gather(p0, b0)
        e10 = lshift(e1,0)
        z2 = e1 - e10
        r = r*2
        if r > n:
            break
        b01 = 1 @ b0
        (b1, tmp) = LOUDS_contract(b01)
        b0 = b1[1:]
        s0 = LOUDS_Duplicate(z2, b0)
    return z2

def LOUDS_sum_children(wc, b):
    n = len(b)//2
    sigma = StableSort(b)
    wc0 = wc @ ([0]*n)
    wb = wc0.AppPerm(sigma)
    s = SuffixSum(wb)
    e = LOUDS_gather(s, b)
    e0 = lshift(e, 0)
    z = e - e0
    return z

def LOUDS_TreeSum_supereasy(b_, w_):
    n = len(b_)//2
    r = 1
    b = b_.dup()
    w = w_.dup()
    while r <= n:
        b1 = b[1:]
        wc = LOUDS_sum_children(w, b1)
        w += wc
        (b, tmp) = LOUDS_contract(b)
        r = r*2
    return w

def LOUDS_depth(b):
    n = len(b)//2
    w = Share([1]*n, b.order())
    ans = LOUDS_PathSum(b, w)
    return ans

def LOUDS_treesize(b):
    n = len(b)//2
    w = Share([1]*n, b.order())
    ans = LOUDS_TreeSum(b, w)
    return ans

def LOUDS_ancestors(b):
    n = len(b)//2
    w = Share([0]*n, b.order())
    w = _const(n, 0, order(b))
    i = 0
    while i < n:
        w[i] = i+1
    ans = LOUDS_PathSum(b, w)
    return ans

def LOUDS_degree(b):
    n = len(b)//2
    wc = Share([1]*n, b.order())
    ans = LOUDS_sum_children(wc, b)
    return ans

def LOUDS_prerank(bb):
    n = len(bb)//2
    b = bb[1:]
    sigma = StableSort(b)
    d = LOUDS_depth(bb)
    e = d[1:]
    e2 = d[:-1]
    e -= e2
    e = 1 @ e
    f = e.dup()
    s = LOUDS_treesize(bb)
    f0 = Share([0]*n, e.order())
    f0 = f0 @ f
    f0 = f0 @ e
    f1 = f0.AppPerm(sigma) # 各レベルの最初の位置
    s1 = Share([0]*n, s.order())
    s2 = s @ s1
    s0 = s2.AppPerm(sigma)
    bs = PrefixSum(~b) # 子の ID
    zeros = Share([0]*(n*2), bs.order())
    h1 = IfThenElse(f1, bs, zeros) # 各レベルの最後のノードの ID
    p = PrefixSum(s0)
    f1 = 1 @ f1
    p0 = p - h1
    p0 = 0 @ p0
    pp = Propagate(f1, p0)
    pp = pp[1:]
    q = p - pp
    q = q - s0
    q = q - PrefixSum(h1)
    x = q.AppInvPerm(sigma)
    x = x[0:n]
    a = LOUDS_ancestors(bb)
    z = x + a
    z = z - Perm_ID(z)
    return z

def LOUDS_to_BP(bb):
    n = len(bb)//2
    b = bb[1:]
    sigma = StableSort(b)
    d = LOUDS_depth(bb)
    s = LOUDS_treesize(bb)
    z = LOUDS_prerank(bb)
    v = z * 2
    c = Share([1]*n, v.order())
    v = v - c
    v = v - d #  開き括弧の位置
    w = s * 2
    w = w - c
    w = w + v #  閉じ括弧の位置
    vw = v @ w
    pi = vw.AppPerm(sigma)
    bp = b.AppInvPerm(pi)
    return (bp, pi)

def BP_to_LOUDS(BP):
    OP = 0
    CP = 1
    n = len(BP)
    k = blog(n-1)+1
    P = BP.extend(1<<k)
#    print('P0')
#    P.print()
    P = -2 * P
#    print('P0')
#    P.print()
    i = 0
    while i < n:
        P[i] += 1
        i += 1
#    print('P1')
#    P.print()
    D = 0 @ PrefixSum(P)
    BP0 = OP @ BP

#    print('D')
#    D.print()
    (tmp, sigma) = radix_sort(D)
    print('sigma')
    sigma.print()
    LOUDS = BP0.AppPerm(sigma)
    LOUDS = ~LOUDS
    ans = (LOUDS, sigma)
    return ans

def BP_to_share(BP):
    OP = 0
    CP = 1
    n = len(BP)
    s = Share([0]*n, 2)
    i = 0
    while i < n:
        if BP[i] == '(':
            c = OP
        elif BP[i] == ')':
            c = CP
        else:
            print('BP[', i, '] = ', BP[i])
            return None
        s[i] = c
        i += 1
    return s



import sys
import random
from csclib import *



if __name__ == '__main__':
  party = -1

  args = sys.argv
  if len(args) >= 2:
    party = int(args[1])

  print('party ', party)
  Csclib_start(party)

  str = "tobeornottobe"
  T = []
  for c in str:
    T.append(ord(c))
  sT = Share(T, 256)
#  (I, LCP) = SuffixSort_LCP(sT)
#  print("I ")
#  I.print()
#  print("LCP ")
#  LCP.print()
  SuffixSort_DC3(sT)



import sys
import random
import time
from csclib import *



if __name__ == '__main__':
  party = -1

  args = sys.argv
  if len(args) >= 3:
    party = int(args[1])
    fname = args[2]
  else:
    fname = args[1]

  print('party ', party)
  Csclib_start(party)

#  str = "tobeornottobe"
#  str = "tobeorno"
#  str = "tobeornottob"
  f = open(fname, 'r')
  str = f.read()
  f.close()

  T = []
  for c in str:
    T.append(ord(c))
  sT = Share(T, 256)
#  (I, LCP) = SuffixSort_LCP(sT)
#  print("I ")
#  I.print()
#  print("LCP ")
#  LCP.print()

  time_start = time.time()
  SA = SuffixSort(sT)
  time_end = time.time()
  lap = time_end- time_start
  print('SuffixSort    time ', lap)

  time_start = time.time()
  SAd = SuffixSort_DC3(sT)
  time_end = time.time()
  lap = time_end- time_start
  print('SuffixSortDC3 time ', lap)

  SA0 = SA.get()
  SAd0 = SAd.get()
  print(SA0 == SAd0)
#  print('SA0 ', SA0)
#  print('SAd0 ', SAd0)
#  Suffix_print(T, SA0)



# setup.py
from setuptools import setup, Extension, find_packages

import shutil, glob, os

#c_dir = 'src/csclib'
#os.makedirs(c_dir, exist_ok = True)
#
#for filename in glob.glob('../../c/*.h'):
#  print('copy', filename, 'to', c_dir)
#  shutil.copy(filename, c_dir)
#shutil.copy('../../README.md', '.')

setup(name = 'csclib',
      ext_modules = [
          Extension(
              name = 'ccsclib',
              sources = ['src/csclib/pycsclib.c'],
              extra_compile_args=['-O3', '-Wno-unused-function', '-Wno-unused-variable'],
#              extra_compile_args=['-g', '-Wno-unused-function', '-Wno-unused-variable'],
           )],
      package_data={"csclib": ["*.h"]}
)



def blog(x):
    l = -1
    while x > 0:
        x >>= 1
        l += 1
    return l

def dup(v):
  n = len(v)
  ans = [0] * n
  i = 0
  while i < n:
    ans[i] = v[i]
    i += 1
  return ans

def Sum(v):
    n = len(v)
    ans = v[0]
    i = 1
    while i < n:
        ans += v[i]
        i += 1
    return ans

def rank0(v):
    n = len(v)
    ans = [0] * n
    sum = 0
    i = 0
    while i < n:
        sum += 1 - v[i]
        ans[i] = sum
        i += 1
    return ans

def rank1(v):
    n = len(v)
    ans = [0] * n
    sum = 0
    i = 0
    while i < n:
        sum += v[i]
        ans[i] = sum
        i += 1
    return ans

def rshift(v, z):
    n = len(v)
    ans = [z] + v[:-1]
    return ans

def vneg(v):
  n = len(v)
  i = 0
  ans = [0] * n
  while i < n:
    ans[i] = 1 - v[i]
    i += 1
  return ans

def IfThenElse(f, a, b):
    n = len(f)
    i = 0
    ans = [0] * n
    while i < n:
      ans[i] = (a[i] - b[i]) * f[i] + b[i]
      i += 1
    return ans

def vadd(x, y):
  n = len(x)
  ans = [0] * n
  i = 0
  while i < n:
    ans[i] = x[i] + y[i]
    i += 1
  return ans

def select1(g):
    n = len(g)
    s0 = rank0(g)
    s0 = rshift(s0, 0)
    m = Sum(g)
    i = 0
    while i < n:
        s0[i] += m
        i += 1
    s1 = rank1(g)
    s1 = rshift(s1, 0)
    sigma = IfThenElse(g, s1, s0)
    gneg = vneg(g)
#    t0 = n * gneg
    t0 = [0] * n
    i = 0
    while i < n:
      t0[i] = gneg[i] * n
      i += 1
    t1 = dup(g)
    i = 0
    while i < n:
        t1[i] = t1[i] * i
        i += 1
    t = vadd(t0, t1)
    u = t.AppInvPerm(sigma)
    return u


if __name__ == '__main__':

  B = [0,1,0,0,1,1,0,0,1,0,0,0,0,1,0,0]

  n = len(B)
  m = Sum(B)[0]
  print(n, m)

  w = blog(n-1)+1
  z = blog(m)
  print(w, z)

  s1 = select1(B)[:m]
  s1.print()

#  sb = s1.A2B(2).get()
#  R = sb[:w-z]
#  Q = sb[w-z:]
#  i = 0
#  while i < len(R):
#    R[i] = R[i].extend(1<<(w-z))
#    i += 1
#  i = 0
#  while i < len(Q):
#    Q[i] = Q[i].extend(1<<z)
#    i += 1
#  rb = Bits().new(R)
#  qb = Bits().new(Q)
#
#  L = rb.B2A()
#  q = qb.B2A()
  (Q, L) = partition(s1, w-z)
  L.print()
  Q.print()

  U = Q.Unary(1<<z)
  H = U.A2QB(1<<z, 2)[1]
  H.print()
  L.print()
